#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include "version.h"
#include "notescheck.h"

#define PROGRAMTYPE 1

/* b:          counter for finding the semicolon before the EBT notes id
   c:          counter for all notes
   ebtidstart: first position of the EBT notes id
   en:         for the new Europa series of banknotes, checks if second position in the serial
               number is a digit or not (Europa series should have no digit at this position)
   f:          combination check, at the end: f=1 means valid combination
   fn:         maximum fifth note character in ASCII code, normally it is J, so this value is 74
   i:          for loops, etc
   ms:         maximum sixth note character in ASCII code, normally it is 6, so this value is 54
   n:          value of the note
   t:          first position of the short code
   sc:         correct short value: f=1 means NOT correct short code value
   sn:         beginning position in the string of the serial number
   fcount:     counter unknown combinations
   scount:     semicolons per line (if there are no extra semicolons in the comment, it should be 13)
   tcount:     counter for typing error in the short code value
   y:          for posistion of the year in the CSV value
   year:       the int value of the year of the note, after the first semicolon in the CSV file */

int b=0, c=0, ebtidstart=0, f=0, fn=74, i=0, ms=54, n=0, t=0; /* , y=0; we might need this later*/
int sc=0, sn=0, en=0;
int fcount=0, scount=0, tcount=0; /* , year=0; we might need this later */

/* a buffer, read it line after line from CSV file */
char puffer[1024];

int main( int argc, char *argv[] )
 {
  /* this points to the CSV file */
  FILE *quelle;

  /* these four long ints are used for time calculation - how long does it take to check the notes */
  long int time1, time2, thou, sec;

  /* start time */
  time1 = clock();

  printf("\nEBT Notes Check version ");
  printf("%s by marty44 from %s", PROGRAMVERSION, PROGRAMDATE);
  printf("\n\nprinter short codes are taken from\nhttp://liste.eurobillets.free.fr/ (updated %s", LISTDATE);
  printf(")\nyou can reach me via the EBT forum (username marty44)\n");
  printf("thread about EBT Notes Check:\nhttps://forum.eurobilltracker.com/viewtopic.php?f=5&t=21676\n\n");
  printf("--> Please try my online tool too: EBTCheck, see https://ebtcheck.marty44.net/ebtcheck/\n");
  printf("--> It checks your CSV file too and is able to generate some statistics.\n");
  printf("--> Example: https://ebtcheck.marty44.net/ebtcheck/user23/\n\n");

  /* the command line must have 2 arguments (filename of the program and the CSV file) */
  if (argc != 2)
   {
    printf("usage: %s filename\n\n", argv[0]);
    exit(EXIT_FAILURE);
   }

  /* check if we can open the CSV file */
  if((quelle = fopen(argv[1], "r")) == NULL)
   {
    printf("Could not open %s\n\n", argv[1]);
    return EXIT_FAILURE;
   }

  /* get the first line and check if it is a valid CSV file */
  fgets(puffer, 1024, quelle);
  for(i=0; i<20; i++)
   {
    if(puffer[i] == 'E') { t=i; break; }
   }
  if(!((puffer[t+1] == 'B') && (puffer[t+10] == 'v') && (puffer[t+11] == '2')))
   {
    printf("%s doesn't look like a EBT notes CSV file. Abort.\n\n", argv[1]);
    fclose(quelle);
    return EXIT_FAILURE;
   }

  printf("--------------------------------------------------------------------------------------------------------------------------------\n");

  /* read in line via line */
  while(fgets(puffer, 1024, quelle))
   {
    /* call the notescheck subroutine */
    notescheck(PROGRAMTYPE);

    /* if it is a Europa series note and there is at the second position in the serial number a digit, it is not correct */
    if(en==1)
     {
      printf("It looks like this note is an Europa series note, but there is a digit at the second position in the serial number.\n");
      f=0;
     }

    /* if it is no valid combination, print it */
    if(f==0)
     {
      fcount++;
      printf("unknown combination:                      %3d Euro note ", n);
      printf("%c%c%c%c%c%c ", puffer[t], puffer[t+1], puffer[t+2], puffer[t+3], puffer[t+4], puffer[t+5]);
      printf("%c%c%c%c%c%c%c%c%c%c%c%c ", puffer[sn], puffer[sn+1], puffer[sn+2], puffer[sn+3], puffer[sn+4], puffer[sn+5], puffer[sn+6], puffer[sn+7], puffer[sn+8], puffer[sn+9], puffer[sn+10], puffer[sn+11]);
      printf("EBT notes id: ");
      ebtidstart=0;
      b=0;
      /* find the position of the semicolon before the ebt notes id */
      for(i=0; i<1024; i++)
       {
        if(puffer[i] == ';') b++;
        if(b == ( scount-4 )) { ebtidstart=i; break; }
       }
      ebtidstart++;
      for(i=0; i<15; i++)
       {
        printf("%c", puffer[ebtidstart+i]);
        if(puffer[ebtidstart+i+1] == ';') break;
       }
      printf("\n");
     }

    /* if the short code value is not correct, print it */
    if(sc==1)
     {
      printf("printer short code seems to be wrong      %3d Euro note ", n);
      printf("%c%c%c%c%c%c ", puffer[t], puffer[t+1], puffer[t+2], puffer[t+3], puffer[t+4], puffer[t+5]);
      printf("%c%c%c%c%c%c%c%c%c%c%c%c ", puffer[sn], puffer[sn+1], puffer[sn+2], puffer[sn+3], puffer[sn+4], puffer[sn+5], puffer[sn+6], puffer[sn+7], puffer[sn+8], puffer[sn+9], puffer[sn+10], puffer[sn+11]);
      printf("EBT notes id: ");
      ebtidstart=0;
      b=0;
      /* find the position of the semicolon before the ebt notes id */
      for(i=0; i<1024; i++)
       {
        if(puffer[i] == ';') b++;
        if(b == ( scount-4 )) { ebtidstart=i; break; }
       }
      ebtidstart++;
      for(i=0; i<15; i++)
       {
        printf("%c", puffer[ebtidstart+i]);
        if(puffer[ebtidstart+i+1] == ';') break;
       }
      printf("\n");
      tcount++;
     }

    /* check if there is a not valid character in the short code
       example: L018K1 is not correct (highest allowed letter at the fifth position is J)
                L017I2 is not correct (highest allowed letter at the fifth position is H */
    if((int) puffer[t+4] > fn)
     {
      printf("fifth character of short code looks wrong %3d Euro note ", n);
      printf("%c%c%c%c%c%c ", puffer[t], puffer[t+1], puffer[t+2], puffer[t+3], puffer[t+4], puffer[t+5]);
      printf("%c%c%c%c%c%c%c%c%c%c%c%c", puffer[sn], puffer[sn+1], puffer[sn+2], puffer[sn+3], puffer[sn+4], puffer[sn+5], puffer[sn+6], puffer[sn+7], puffer[sn+8], puffer[sn+9], puffer[sn+10], puffer[sn+11]);
      if((sc==1) || (f==0)) { printf("   !!!!!! please take also care one line above !!!!!!"); }
      else
       {
        tcount++;
        printf(" EBT notes id: ");
        ebtidstart=0;
        b=0;
        /* find the position of the semicolon before the ebt notes id */
        for(i=0; i<1024; i++)
         {
          if(puffer[i] == ';') b++;
          if(b == ( scount-4 )) { ebtidstart=i; break; }
         }
        ebtidstart++;
        for(i=0; i<15; i++)
         {
          printf("%c", puffer[ebtidstart+i]);
          if(puffer[ebtidstart+i+1] == ';') break;
         }
       }
      printf("\n                                                            ");
      if(f != 0) printf("\\> highest valid letter at fifth position for this short code is a %c\n", fn);
      else printf("\\> take also care about the fifth position, it might be wrong\n");
     }

    /* check for a valid value of the sixt position in the short code */
    if((int) puffer[t+5] > ms)
     {
      if((int) puffer[t+4] > fn) {}
      else
       {
        printf("sixth character of short code looks wrong %3d Euro note ", n);
        printf("%c%c%c%c%c%c ", puffer[t], puffer[t+1], puffer[t+2], puffer[t+3], puffer[t+4], puffer[t+5]);
        printf("%c%c%c%c%c%c%c%c%c%c%c%c", puffer[sn], puffer[sn+1], puffer[sn+2], puffer[sn+3], puffer[sn+4], puffer[sn+5], puffer[sn+6], puffer[sn+7], puffer[sn+8], puffer[sn+9], puffer[sn+10], puffer[sn+11]);
        if((sc==1) || (f==0)) { printf("   !!!!!! please take also care one line above !!!!!!"); }
        else
         {
          tcount++;
          printf(" EBT notes id: ");
          ebtidstart=0;
          b=0;
          /* find the position of the semicolon before the ebt notes id */
          for(i=0; i<1024; i++)
           {
            if(puffer[i] == ';') b++;
            if(b == ( scount-4 )) { ebtidstart=i; break; }
           }
          ebtidstart++;
          for(i=0; i<15; i++)
           {
            printf("%c", puffer[ebtidstart+i]);
            if(puffer[ebtidstart+i+1] == ';') break;
           }
         }
        printf("\n");
       }
      printf("                                                             ");
      if(f != 0) printf("\\> highest valid value at sixth position for this short code is a %c\n", ms);
      else printf("\\> take also care about the sixth position, it might be wrong\n");
     }
   if((sc==1) || (f==0) || ((int) puffer[t+5] > ms) || ((int) puffer[t+4] > fn))
   printf("--------------------------------------------------------------------------------------------------------------------------------\n");
   }

  /* print a summary */
  printf("\n%d unknown combination", fcount);
  if(fcount != 1) printf("s");
  if(tcount > 0)
   {
    printf(", maybe %d typing error", tcount);
    if(tcount != 1) printf("s");
    printf(" at the printer codes");
   }
  printf(", %d notes total\n\n", c);

  /* time calculation */
  time2 = clock ();
  thou  = (time2-time1) * 1000 / CLOCKS_PER_SEC;
  sec   = thou / 1000;
  thou -= sec * 1000;
  printf ("Checking of notes took %ld.%03ld seconds\n\n", sec, thou);
  fclose(quelle);
  return EXIT_SUCCESS;
 }
