#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include "version.h"
#include "combinations.h"
#include "noteschecksignatures.h"

/*
  combos array - here are the valid combinations stored
  1st = active, 2nd = letter short code value, 3rd = letter serial number value,
                 4th = value of the note, 5th = series, 6th signature
  short code:    A=0,B=1,C=2,D=3,E=4,F=5,G=6,H=7,I=8,J=9,K=10,L=11,M=12,N=13,O=14,P=15,Q=16,
                 R=17,S=18,T=19,U=20,V=21,W=22,X=23,Y=24,Z=25
  serial number: A=0,B=1,C=2,D=3,E=4,F=5,G=6,H=7,I=8,J=9,K=10,L=11,M=12,N=13,O=14,P=15,Q=16,
                 R=17,S=18,T=19,U=20,V=21,W=22,X=23,Y=24,Z=25
  note value:    5=0,10=1,20=2,50=3,100=4,200=5,500=6
  series:        0 = 2002 series, 1 = Europa series
*/
int combos[3][26][26][7][2][4];
int f=0, n=0, us=0;
int x,y,z,s,u;
char shortcodewert[4];

int main( int argc, char *argv[] )
 {
  // this points to the CSV file

  FILE *quelle;

  /* a:              counter for finding the semicolon before the short code
     b:              counter for finding the semicolon before the EBT notes id
     c:              counter for all notes
     ebtidstart:     first position of the EBT notes id
     f:              is set to one, if it is the first note of a combination
     fcomb:          found combinations
     i:              for loops, etc
     n:              value of the note
     completecombos: complete combos that exist
     city:           first position of the city
     land:           first position of the land
     t:              first position of the short code
     sn:             beginning position in the string of the serial number
     scount:         semicolons per line */
  int a=0, b=0, c=0, ebtidstart=0, i=0, t=0;
  int sn=0;
  int scount=0;
  int city=0, land=0;
  int fcomb=0, completecombos=0;

  // these four long ints are used for time calculation - how long does it take to check the notes
  long int time1, time2, thou, sec;

  // a buffer, read it line after line from CSV file
  char puffer[1024];

  // the array for the combinations, used to identify the notes

  // start time
  time1 = clock();

  // fill the combos array with the known combinations - see combinations.h

  fillcombinations();

  printf("\nFirst Notes version %s by marty44 from %s\n\n", PROGRAMVERSION, PROGRAMDATE);
  printf("--> Please try my online tool too: EBTCheck, see http://ebtcheck.marty44.net/ebtcheck/\n");
  printf("--> It checks your CSV file too and is able to generate some statistics.\n");
  printf("--> Example: http://ebtcheck.marty44.net/ebtcheck/user23/\n\n");

  // the command line must have 2 arguments (filename of the program and the CSV file)
  if ( argc != 2 )
   {
    printf("usage: %s filename\n\n", argv[0]);
    exit(EXIT_FAILURE);
   }

  // check if we can open the CSV file
  if((quelle = fopen(argv[1], "r")) == NULL)
   {
    printf("Could not open %s\n\n", argv[1]);
    return EXIT_FAILURE;
   }

  // get the first line and check if it is a valid CSV file
  fgets(puffer, 1024, quelle);
  for(i=0; i<20; i++)
   {
    if(puffer[i] == 'E') { t=i; break; }
   }
  if(!((puffer[t+1] == 'B') && (puffer[t+10] == 'v') && (puffer[t+11] == '2')))
   {
    printf("%s doesn't look like a EBT notes CSV file. Abort.\n\n", argv[1]);
    fclose(quelle);
    return EXIT_FAILURE;
   }

  // read in line via line
  while(fgets(puffer, 1024, quelle))
   {
    // count how many notes we have already checked and give every 100000 notes a status message
    c++;
    if((c%100000) == 0) printf("Status: already %d notes checked\n", c);

    // make sure that all needed variables are set to zero
    a=0; f=0; i=0; n=0; s=0; t=0; x=0; y=0;
    scount=0;
    city=0, land=0;
    shortcodewert[0]='0'; shortcodewert[1]='0'; shortcodewert[2]='0'; shortcodewert[3]='\n';

    /* find out how many semicolons the current line has and store it into scount
       it could be that in the note comment a semicolon exist so we need them all */
    for(i=0; i<1024; i++)
     {
      if(puffer[i] == '\0') break;
      if(puffer[i] == ';') scount++;
     }
    
    /* identify what value the note has and store it into n
       store in sn the beginning position of the serial number */
    if((puffer[0] == '5') && (puffer[1] == ';')) { sn=7; n=0; }
    if((puffer[0] == '1') && (puffer[2] == ';')) { sn=8; n=1; }
    if((puffer[0] == '2') && (puffer[2] == ';')) { sn=8; n=2; }
    if((puffer[0] == '5') && (puffer[2] == ';')) { sn=8; n=3; }
    if((puffer[0] == '1') && (puffer[3] == ';')) { sn=9; n=4; }
    if((puffer[0] == '2') && (puffer[3] == ';')) { sn=9; n=5; }
    if((puffer[0] == '5') && (puffer[3] == ';')) { sn=9; n=6; }

    // find out the position of the semicolon before the short code and store it into t
    for(i=sn; i<1024; i++)
     {
      if(puffer[i] == ';') a++;
      if(a == ( scount-7 )) { t=i; break; }
     }

    // increment t so we have now the first sign of the short code
    t++;
    
    /* move the 3 digits into shortcodewert
        example: P015A1 = 015 */
    shortcodewert[0]=puffer[t+1];
    shortcodewert[1]=puffer[t+2];
    shortcodewert[2]=puffer[t+3];

    // check if it the first note, if so store it in appropriate combos field
    x = (int) puffer[t] - 65;
    y = (int) puffer[sn] - 65;
    if(isdigit(puffer[sn+1])) s=0;
     else s=1;

    for(u=0;u<4;u++)
     {
      if(combos[1][x][y][n][s][u] == 1)
       {
        if(combos[0][x][y][n][s][u] != 1)
         {
          noteschecksignatures();
          if(x == 10 && y == 19 && n == 1 && s == 0 && u != 0 && (atoi(shortcodewert) == 7)) notescheck10KT(puffer[sn+1]); // check 10 K007/T if JCT or MD
          if(x == 5 && y == 13 && n == 2 && s == 0 && u != 2 && (atoi(shortcodewert) == 2)) notescheck20FN(puffer[sn+3], puffer[sn+4], puffer[sn+5], puffer[sn+6]); // check 20 F002/N if WD or JCT
          if(x == 5 && y == 13 && n == 4 && s == 0 && u != 2 && (atoi(shortcodewert) == 2)) notescheck100FN(puffer[sn+3], puffer[sn+4], puffer[sn+5]); // check 100 F002/N if WD or JCT
          if(x == 15 && y == 23 && n == 1 && s == 0 && u != 2 && (atoi(shortcodewert) == 5)) notescheck10PX(puffer[sn+1], puffer[sn+2], puffer[sn+3]); // check 10 P005/X if WD or JCT
          if(x == 17 && y == 17 && n == 2 && s == 1 && (atoi(shortcodewert) == 13)) notescheck20EuropaRR(puffer[sn+1]); // check 20 Europa R013/R if MD or CL
          if(x == 18 && y == 18 && n == 3 && s == 1 && ( (atoi(shortcodewert) == 36) || (atoi(shortcodewert) == 37) || (atoi(shortcodewert) == 41))) notescheck50EuropaSS(puffer[sn+3], puffer[sn+4], puffer[sn+5], puffer[sn+6], puffer[sn+7], puffer[sn+8]); // check 50 Europa S036,37,41/S if MD or CL
          if(x == 21 && y == 21 && n == 1 && s == 1 && (atoi(shortcodewert) == 11)) notescheck10EuropaVV(puffer[sn+2], puffer[sn+3]); // check 10 Europa V011/V if MD or CL
          if(x == 21 && y == 21 && n == 3 && s == 1 && (atoi(shortcodewert) == 18)) notescheck50EuropaVV(puffer[sn+2], puffer[sn+3], puffer[sn+4]); // check 50 Europa V018/V if MD or CL
          if(x == 24 && y == 24 && n == 3 && s == 1 && (atoi(shortcodewert) == 3)) notescheck50EuropaYY(puffer[sn+2], puffer[sn+3], puffer[sn+4], puffer[sn+5], puffer[sn+6]); // check 50 Europa Y003/Y if MD or CL
         }
       }
     }
     
    // if f is one, it is a first note
    if(f==1)
     {
      if(n==5) n=200; if(n==0) n=5; if(n==1) n=10; if(n==2) n=20; if(n==3) n=50; if(n==4) n=100; if(n==6) n=500;
      printf("%7d. note was the first %3d %c/%c ", c, n, puffer[t], puffer[sn]);
      if(us==0) printf(" WD");
      if(us==1) printf("JCT");
      if(us==2) printf(" MD");
      if(us==3) printf(" CL");
      printf(" ");
      if(s==0) printf("      ");
       else printf("Europa");
      printf(" note, entered in ");

      a=0;
      // locate the first position of the city
      for(i=sn; i<1024; i++)
       {  
        if(puffer[i] == ';') a++;
        if(a == ( scount-10 )) { city=i; break; }
       }
      city++;
      // print the city name
      for(i=city; i<1024; i++)
       {
        if(puffer[i] == ';') break;
        printf("%c", puffer[i]);
       }
      // locate the first position of the country
      a=0;
      for(i=sn; i<1024; i++)
       {
        if(puffer[i] == ';') a++;
        if(a == ( scount-9 )) { land=i; break; }
       }
      land++;
      // print the country name
      printf(", ");
      for(i=land; i<1024; i++)
       {
        if(puffer[i] == ';') break;
        printf("%c", puffer[i]);
       }
      printf(", EBT note id: ");
      ebtidstart=0;
      b=0;
      // find the position of the semicolon before the ebt notes id
      for(i=0; i<1024; i++)
       {
        if(puffer[i] == ';') b++;
        if(b == ( scount-4 )) { ebtidstart=i; break; }
       }
      ebtidstart++;
      for(i=0; i<15; i++)
       {
        printf("%c", puffer[ebtidstart+i]);
        if(puffer[ebtidstart+i+1] == ';') break;
       }
      printf("\n");
     }
   }

  // print a summary
  printf("\n%d notes total, ", c);
  for(x=0;x<26;x++)
   {
    for(y=0;y<26;y++)
     {
      for(z=0;z<7;z++)
       {
        for(s=0;s<2;s++)
         {
          for(u=0;u<4;u++)
           {
            if(combos[1][x][y][z][s][u]==1)
             {
              completecombos++;
              if(combos[0][x][y][z][s][u]==1) fcomb++;
             }
           }
         }
       }
     }
   }
  printf("%d of %d combinations found\n\n", fcomb, completecombos);
// should be now ok since 15 Jan 2023 printf("recognition of Europa 50 S036/S, S037/S and S041/S is not yet implemented\n\n");
  if(fcomb < completecombos)
   {
    printf("%d missing combination", completecombos-fcomb);
    if((completecombos-fcomb) != 1) printf("s");
    printf(":");
    i=0;
    for(x=0;x<26;x++)
     {
      for(y=0;y<26;y++)
       {
        for(z=0;z<7;z++)
         {
          for(s=0;s<2;s++)
           {
            for(u=0;u<4;u++)
             {
              if(combos[1][x][y][z][s][u]==1)
               {
                if(combos[0][x][y][z][s][u]==0)
                 {
                  n=0;
                  if(z==0) n=5;
                  if(z==1) n=10;
                  if(z==2) n=20;
                  if(z==3) n=50;
                  if(z==4) n=100;
                  if(z==5) n=200;
                  if(z==6) n=500;
                  if(i==1) printf(",");
                  i=1;
                  printf(" %d %c/%c ", n, x+65, y+65);
                  if(u==0) printf("WD");
                  if(u==1) printf("JCT");
                  if(u==2) printf("MD");
                  if(u==3) printf("CL");
                  if(s==1) printf(" Europa");
                 }
               }
             }
           }
         }
       }
     }
    printf("\n\n");
   }

  printf("      Series   WD   JCT    MD    CL   all\n");

  tablecolumn(0,0);
  tablecolumn(0,1);
  tablecolumn(1,0);
  tablecolumn(1,1);
  tablecolumn(2,0);
  tablecolumn(2,1);
  tablecolumn(3,0);
  tablecolumn(3,1);
  tablecolumn(4,0);
  tablecolumn(4,1);
  tablecolumn(5,0);
  tablecolumn(5,1);
  tablecolumn(6,0);

  x=countseriesnotes(0);
  y=countseriesnotesavailable(0);
  printf("\n       2002  series notes: %3d/%3d", x, y);
  if(x==y) printf("   Congratulation!");
  x=countseriesnotes(1);
  y=countseriesnotesavailable(1);
  printf("\n      Europa series notes: %3d/%3d", x, y);
  if(x==y) printf("   Congratulation!");
  if(fcomb==completecombos) printf("\n\nWow you found them all! Report this into this thread:\nhttp://forum.eurobilltracker.com/viewtopic.php?f=5&t=4665\n");
  // time calculation
  time2 = clock ();
  thou  = (time2-time1) * 1000 / CLOCKS_PER_SEC;
  sec   = thou / 1000;
  thou -= sec * 1000;
  printf ("\n\nCheck of notes took %ld.%03ld seconds\n\n", sec, thou);
  fclose(quelle);
  return EXIT_SUCCESS;
 }
